// Vec4.hpp: 4D Vector
//			 This class contains the definition and implementation
//			 of 4 dimensional vectors. Contains overloaded operators
//			 for most vector operations, plus additional methods 
//			 for normalization and length
//////////////////////////////////////////////////////////////////////

#ifndef _VEC4_H_
#define _VEC4_H_

#include <math.h>

#define sqr(x)	((x)*(x))

class Vec4
{
public:
	// 4D Vector representation
	double x, y, z, w;

	///////////////////////////////////
	// Constructors/Set Methods
	///////////////////////////////////
	Vec4() 
	{ x = 0.0f; y = 0.0f; z = 0.0f; w = 0.0f; };
	
	Vec4(const double X, const double Y, const double Z, const double W) 
	{ x = X; y = Y; z = Z; w = W; };

	void Set(const double X, const double Y, const double Z, const double W)
	{ x = X; y = Y; z = Z; w = W; };

	void Set(const Vec4 &V)
	{ x = V.x; y = V.y; z = V.z; w = V.w; };

	////////////////////////////////////
	// 4D Vector Operations
	////////////////////////////////////
	 
	// COMPARISON (==)
    int operator == (const Vec4& A)              
      { return x==A.x && y==A.y && z==A.z && w==A.w; } 

	// ASSIGNMENT (=)
    Vec4& operator = (const Vec4& A)            
      { x=A.x; y=A.y; z=A.z; w=A.w; 
        return(*this);  };

	// ADDITION (+)
    Vec4 operator + (const Vec4& A) const       
      { Vec4 Sum(x+A.x, y+A.y, z+A.z, w+A.w); 
        return(Sum); };

	// SUBTRACTION (-)
    Vec4 operator - (const Vec4& A) const       
      { Vec4 Diff(x-A.x, y-A.y, z-A.z, w-A.w);
        return(Diff); };

	// DOT-PRODUCT (*)
    double operator * (const Vec4& A) const       
      { double DotProd = x*A.x+y*A.y+z*A.z+w*A.w; 
        return(DotProd); };

	// MULTIPLY BY SCALAR (*)
    Vec4 operator * (const double s) const        
      { Vec4 Scaled(x*s, y*s, z*s, w*s); 
        return(Scaled); };

	// DIVIDE BY SCALAR (/)
    Vec4 operator / (const double s) const        
      { Vec4 Scaled(x/s, y/s, z/s, w/s);
        return(Scaled); };

	// SCALAR MULT s*V
    friend inline Vec4 operator *(double s, const Vec4& v)  
      { return Vec4(v.x*s, v.y*s, v.z*s, v.w*s); }

	// ACCUMULATED VECTOR ADDITION (+=)
    void operator += (const Vec4 A)              
      { x+=A.x; y+=A.y; z+=A.z; w+=A.w; };

	// ACCUMULATED VECTOR SUBTRACTION (+=)
    void operator -= (const Vec4 A)              
      { x-=A.x; y-=A.y; z-=A.z; w-=A.w; };

	// ACCUMULATED SCALAR MULT (*=)
    void operator *= (const double s)              
      { x*=s; y*=s; z*=s; w*=s; };

	// ACCUMULATED SCALAR DIV (/=)
    void operator /= (const double s)              
      { x/=s; y/=s; z/=s; w/=s; };

	// NEGATION (-)
    Vec4 operator - (void) const                 
      { Vec4 Negated(-x, -y, -z, -w);
        return(Negated); };

	// Length of Vector
	double Length ()
	{ return ((double)sqrt(sqr(x)+sqr(y)+sqr(z)+sqr(w))); };

	// normalizes the vectors
	void Normalize ()
	{ double L = Length();               
      if (L>0) { x/=L; y/=L; z/=L; w/=L; } };
};


#endif
